@@ -29,10 +29,9 @@ import itertools
 import math
 import os
 import subprocess
-import sys
 from enum import IntEnum
 from functools import cached_property
-from typing import IO, TYPE_CHECKING, Any, List, Literal, NamedTuple, Union
+from typing import IO, Any, Literal, NamedTuple, Union, cast
 
 from . import (
     Image,
@@ -46,9 +45,12 @@ from . import (
 from ._binary import i16le as i16
 from ._binary import o8
 from ._binary import o16le as o16
+from ._util import DeferredError
 
+TYPE_CHECKING = False
 if TYPE_CHECKING:
     from . import _imaging
+    from ._typing import Buffer
 
 
 class LoadingStrategy(IntEnum):
@@ -67,7 +69,7 @@ LOADING_STRATEGY = LoadingStrategy.RGB_AFTER_FIRST
 
 
 def _accept(prefix: bytes) -> bool:
-    return prefix[:6] in [b"GIF87a", b"GIF89a"]
+    return prefix.startswith((b"GIF87a", b"GIF89a"))
 
 
 ##
@@ -103,7 +105,6 @@ class GifImageFile(ImageFile.ImageFile):
 
         self.info["version"] = s[:6]
         self._size = i16(s, 6), i16(s, 8)
-        self.tile = []
         flags = s[10]
         bits = (flags & 7) + 1
 
@@ -155,7 +156,7 @@ class GifImageFile(ImageFile.ImageFile):
         if not self._seek_check(frame):
             return
         if frame < self.__frame:
-            self.im = None
+            self._im = None
             self._seek(0)
 
         last_frame = self.__frame
@@ -168,6 +169,8 @@ class GifImageFile(ImageFile.ImageFile):
                 raise EOFError(msg) from e
 
     def _seek(self, frame: int, update_image: bool = True) -> None:
+        if isinstance(self._fp, DeferredError):
+            raise self._fp.ex
         if frame == 0:
             # rewind
             self.__offset = 0
@@ -258,7 +261,7 @@ class GifImageFile(ImageFile.ImageFile):
                     # application extension
                     #
                     info["extension"] = block, self.fp.tell()
-                    if block[:11] == b"NETSCAPE2.0":
+                    if block.startswith(b"NETSCAPE2.0"):
                         block = self.data()
                         if block and len(block) >= 3 and block[0] == 1:
                             self.info["loop"] = i16(block, 1)
@@ -320,18 +323,20 @@ class GifImageFile(ImageFile.ImageFile):
             else:
                 self._mode = "L"
 
-            if not palette and self.global_palette:
+            if palette:
+                self.palette = palette
+            elif self.global_palette:
                 from copy import copy
 
-                palette = copy(self.global_palette)
-            self.palette = palette
+                self.palette = copy(self.global_palette)
+            else:
+                self.palette = None
         else:
             if self.mode == "P":
                 if (
                     LOADING_STRATEGY != LoadingStrategy.RGB_AFTER_DIFFERENT_PALETTE_ONLY
                     or palette
                 ):
-                    self.pyaccess = None
                     if "transparency" in self.info:
                         self.im.putpalettealpha(self.info["transparency"], 0)
                         self.im = self.im.convert("RGBA", Image.Dither.FLOYDSTEINBERG)
@@ -345,12 +350,15 @@ class GifImageFile(ImageFile.ImageFile):
             if self._frame_palette:
                 if color * 3 + 3 > len(self._frame_palette.palette):
                     color = 0
-                return tuple(self._frame_palette.palette[color * 3 : color * 3 + 3])
+                return cast(
+                    tuple[int, int, int],
+                    tuple(self._frame_palette.palette[color * 3 : color * 3 + 3]),
+                )
             else:
                 return (color, color, color)
 
         self.dispose = None
-        self.dispose_extent = frame_dispose_extent
+        self.dispose_extent: tuple[int, int, int, int] | None = frame_dispose_extent
         if self.dispose_extent and self.disposal_method >= 2:
             try:
                 if self.disposal_method == 2:
@@ -377,7 +385,7 @@ class GifImageFile(ImageFile.ImageFile):
                     self.dispose = Image.core.fill(dispose_mode, dispose_size, color)
                 else:
                     # replace with previous contents
-                    if self.im is not None:
+                    if self._im is not None:
                         # only dispose the extent in this frame
                         self.dispose = self._crop(self.im, self.dispose_extent)
                     elif frame_transparency is not None:
@@ -405,7 +413,7 @@ class GifImageFile(ImageFile.ImageFile):
                 elif self.mode not in ("RGB", "RGBA"):
                     transparency = frame_transparency
             self.tile = [
-                (
+                ImageFile._Tile(
                     "gif",
                     (x0, y0, x1, y1),
                     self.__offset,
@@ -435,7 +443,14 @@ class GifImageFile(ImageFile.ImageFile):
                 self.im = Image.core.fill("P", self.size, self._frame_transparency or 0)
                 self.im.putpalette("RGB", *self._frame_palette.getdata())
             else:
-                self.im = None
+                self._im = None
+        if not self._prev_im and self._im is not None and self.size != self.im.size:
+            expanded_im = Image.core.fill(self.im.mode, self.size)
+            if self._frame_palette:
+                expanded_im.putpalette("RGB", *self._frame_palette.getdata())
+            expanded_im.paste(self.im, (0, 0) + self.im.size)
+
+            self.im = expanded_im
         self._mode = temp_mode
         self._frame_palette = None
 
@@ -453,9 +468,23 @@ class GifImageFile(ImageFile.ImageFile):
             return
         if not self._prev_im:
             return
+        if self.size != self._prev_im.size:
+            if self._frame_transparency is not None:
+                expanded_im = Image.core.fill("RGBA", self.size)
+            else:
+                expanded_im = Image.core.fill("P", self.size)
+                expanded_im.putpalette("RGB", "RGB", self.im.getpalette())
+                expanded_im = expanded_im.convert("RGB")
+            expanded_im.paste(self._prev_im, (0, 0) + self._prev_im.size)
+
+            self._prev_im = expanded_im
+            assert self._prev_im is not None
         if self._frame_transparency is not None:
-            self.im.putpalettealpha(self._frame_transparency, 0)
-            frame_im = self.im.convert("RGBA")
+            if self.mode == "L":
+                frame_im = self.im.convert_transparent("LA", self._frame_transparency)
+            else:
+                self.im.putpalettealpha(self._frame_transparency, 0)
+                frame_im = self.im.convert("RGBA")
         else:
             frame_im = self.im.convert("RGB")
 
@@ -464,7 +493,7 @@ class GifImageFile(ImageFile.ImageFile):
 
         self.im = self._prev_im
         self._mode = self.im.mode
-        if frame_im.mode == "RGBA":
+        if frame_im.mode in ("LA", "RGBA"):
             self.im.paste(frame_im, self.dispose_extent, frame_im)
         else:
             self.im.paste(frame_im, self.dispose_extent)
@@ -496,6 +525,7 @@ def _normalize_mode(im: Image.Image) -> Image.Image:
         return im
     if Image.getmodebase(im.mode) == "RGB":
         im = im.convert("P", palette=Image.Palette.ADAPTIVE)
+        assert im.palette is not None
         if im.palette.mode == "RGBA":
             for rgba in im.palette.colors:
                 if rgba[3] == 0:
@@ -505,7 +535,7 @@ def _normalize_mode(im: Image.Image) -> Image.Image:
     return im.convert("L")
 
 
-_Palette = Union[bytes, bytearray, List[int], ImagePalette.ImagePalette]
+_Palette = Union[bytes, bytearray, list[int], ImagePalette.ImagePalette]
 
 
 def _normalize_palette(
@@ -532,16 +562,18 @@ def _normalize_palette(
 
     if im.mode == "P":
         if not source_palette:
-            source_palette = im.im.getpalette("RGB")[:768]
+            im_palette = im.getpalette(None)
+            assert im_palette is not None
+            source_palette = bytearray(im_palette)
     else:  # L-mode
         if not source_palette:
             source_palette = bytearray(i // 3 for i in range(768))
         im.palette = ImagePalette.ImagePalette("RGB", palette=source_palette)
+    assert source_palette is not None
 
-    used_palette_colors: list[int] | None
     if palette:
-        used_palette_colors = []
-        assert source_palette is not None
+        used_palette_colors: list[int | None] = []
+        assert im.palette is not None
         for i in range(0, len(source_palette), 3):
             source_color = tuple(source_palette[i : i + 3])
             index = im.palette.colors.get(source_color)
@@ -554,20 +586,25 @@ def _normalize_palette(
                     if j not in used_palette_colors:
                         used_palette_colors[i] = j
                         break
-        im = im.remap_palette(used_palette_colors)
+        dest_map: list[int] = []
+        for index in used_palette_colors:
+            assert index is not None
+            dest_map.append(index)
+        im = im.remap_palette(dest_map)
     else:
-        used_palette_colors = _get_optimize(im, info)
-        if used_palette_colors is not None:
-            im = im.remap_palette(used_palette_colors, source_palette)
+        optimized_palette_colors = _get_optimize(im, info)
+        if optimized_palette_colors is not None:
+            im = im.remap_palette(optimized_palette_colors, source_palette)
             if "transparency" in info:
                 try:
-                    info["transparency"] = used_palette_colors.index(
+                    info["transparency"] = optimized_palette_colors.index(
                         info["transparency"]
                     )
                 except ValueError:
                     del info["transparency"]
             return im
 
+    assert im.palette is not None
     im.palette.palette = source_palette
     return im
 
@@ -579,7 +616,8 @@ def _write_single_frame(
 ) -> None:
     im_out = _normalize_mode(im)
     for k, v in im_out.info.items():
-        im.encoderinfo.setdefault(k, v)
+        if isinstance(k, str):
+            im.encoderinfo.setdefault(k, v)
     im_out = _normalize_palette(im_out, palette, im.encoderinfo)
 
     for s in _get_global_header(im_out, im.encoderinfo):
@@ -592,7 +630,9 @@ def _write_single_frame(
     _write_local_header(fp, im, (0, 0), flags)
 
     im_out.encoderconfig = (8, get_interlace(im))
-    ImageFile._save(im_out, fp, [("gif", (0, 0) + im.size, 0, RAWMODE[im_out.mode])])
+    ImageFile._save(
+        im_out, fp, [ImageFile._Tile("gif", (0, 0) + im.size, 0, RAWMODE[im_out.mode])]
+    )
 
     fp.write(b"\0")  # end of image data
 
@@ -600,7 +640,10 @@ def _write_single_frame(
 def _getbbox(
     base_im: Image.Image, im_frame: Image.Image
 ) -> tuple[Image.Image, tuple[int, int, int, int] | None]:
-    if _get_palette_bytes(im_frame) != _get_palette_bytes(base_im):
+    palette_bytes = [
+        bytes(im.palette.palette) if im.palette else b"" for im in (base_im, im_frame)
+    ]
+    if palette_bytes[0] != palette_bytes[1]:
         im_frame = im_frame.convert("RGBA")
         base_im = base_im.convert("RGBA")
     delta = ImageChops.subtract_modulo(im_frame, base_im)
@@ -631,7 +674,8 @@ def _write_multiple_frames(
                 for k, v in im_frame.info.items():
                     if k == "transparency":
                         continue
-                    im.encoderinfo.setdefault(k, v)
+                    if isinstance(k, str):
+                        im.encoderinfo.setdefault(k, v)
 
             encoderinfo = im.encoderinfo.copy()
             if "transparency" in im_frame.info:
@@ -655,16 +699,24 @@ def _write_multiple_frames(
                         im_frames[-1].encoderinfo["duration"] += encoderinfo["duration"]
                     continue
                 if im_frames[-1].encoderinfo.get("disposal") == 2:
-                    if background_im is None:
-                        color = im.encoderinfo.get(
-                            "transparency", im.info.get("transparency", (0, 0, 0))
-                        )
-                        background = _get_background(im_frame, color)
-                        background_im = Image.new("P", im_frame.size, background)
-                        background_im.putpalette(im_frames[0].im.palette)
-                    bbox = _getbbox(background_im, im_frame)[1]
+                    # To appear correctly in viewers using a convention,
+                    # only consider transparency, and not background color
+                    color = im.encoderinfo.get(
+                        "transparency", im.info.get("transparency")
+                    )
+                    if color is not None:
+                        if background_im is None:
+                            background = _get_background(im_frame, color)
+                            background_im = Image.new("P", im_frame.size, background)
+                            first_palette = im_frames[0].im.palette
+                            assert first_palette is not None
+                            background_im.putpalette(first_palette, first_palette.mode)
+                        bbox = _getbbox(background_im, im_frame)[1]
+                    else:
+                        bbox = (0, 0) + im_frame.size
                 elif encoderinfo.get("optimize") and im_frame.mode != "1":
                     if "transparency" not in encoderinfo:
+                        assert im_frame.palette is not None
                         try:
                             encoderinfo["transparency"] = (
                                 im_frame.palette._new_color_index(im_frame)
@@ -727,7 +779,8 @@ def _write_multiple_frames(
             if not palette:
                 frame_data.encoderinfo["include_color_table"] = True
 
-            im_frame = im_frame.crop(frame_data.bbox)
+            if frame_data.bbox != (0, 0) + im_frame.size:
+                im_frame = im_frame.crop(frame_data.bbox)
             offset = frame_data.bbox[:2]
         _write_frame_data(fp, im_frame, offset, frame_data.encoderinfo)
     return True
@@ -902,6 +955,7 @@ def _get_optimize(im: Image.Image, info: dict[str, Any]) -> list[int] | None:
             if optimise or max(used_palette_colors) >= len(used_palette_colors):
                 return used_palette_colors
 
+            assert im.palette is not None
             num_palette_colors = len(im.palette.palette) // Image.getmodebands(
                 im.palette.mode
             )
@@ -951,7 +1005,13 @@ def _get_palette_bytes(im: Image.Image) -> bytes:
     :param im: Image object
     :returns: Bytes, len<=768 suitable for inclusion in gif header
     """
-    return im.palette.palette if im.palette else b""
+    if not im.palette:
+        return b""
+
+    palette = bytes(im.palette.palette)
+    if im.palette.mode == "RGBA":
+        palette = b"".join(palette[i * 4 : i * 4 + 3] for i in range(len(palette) // 3))
+    return palette
 
 
 def _get_background(
@@ -964,6 +1024,7 @@ def _get_background(
             # WebPImagePlugin stores an RGBA value in info["background"]
             # So it must be converted to the same format as GifImagePlugin's
             # info["background"] - a global color table index
+            assert im.palette is not None
             try:
                 background = im.palette.getcolor(info_background, im)
             except ValueError as e:
@@ -1055,7 +1116,9 @@ def _write_frame_data(
         _write_local_header(fp, im_frame, offset, 0)
 
         ImageFile._save(
-            im_frame, fp, [("gif", (0, 0) + im_frame.size, 0, RAWMODE[im_frame.mode])]
+            im_frame,
+            fp,
+            [ImageFile._Tile("gif", (0, 0) + im_frame.size, 0, RAWMODE[im_frame.mode])],
         )
 
         fp.write(b"\0")  # end of image data
@@ -1121,18 +1184,9 @@ def getdata(
     class Collector(BytesIO):
         data = []
 
-        if sys.version_info >= (3, 12):
-            from collections.abc import Buffer
-
-            def write(self, data: Buffer) -> int:
-                self.data.append(data)
-                return len(data)
-
-        else:
-
-            def write(self, data: Any) -> int:
-                self.data.append(data)
-                return len(data)
+        def write(self, data: Buffer) -> int:
+            self.data.append(data)
+            return len(data)
 
     im.load()  # make sure raster data is available
 
