@@ -24,8 +24,11 @@ and has been tested with a few sample files found using google.
 """
 from __future__ import annotations
 
+from typing import IO
+
 from . import Image, ImageFile
 from ._binary import i32le as i32
+from ._typing import StrOrBytesPath
 
 
 class WalImageFile(ImageFile.ImageFile):
@@ -50,15 +53,15 @@ class WalImageFile(ImageFile.ImageFile):
         if next_name:
             self.info["next_name"] = next_name
 
-    def load(self):
-        if not self.im:
+    def load(self) -> Image.core.PixelAccess | None:
+        if self._im is None:
             self.im = Image.core.new(self.mode, self.size)
             self.frombytes(self.fp.read(self.size[0] * self.size[1]))
             self.putpalette(quake2palette)
         return Image.Image.load(self)
 
 
-def open(filename):
+def open(filename: StrOrBytesPath | IO[bytes]) -> WalImageFile:
     """
     Load texture from a Quake2 WAL texture file.
 
