@@ -10,11 +10,6 @@ def is_path(f: Any) -> TypeGuard[StrOrBytesPath]:
     return isinstance(f, (bytes, str, os.PathLike))
 
 
-def is_directory(f: Any) -> TypeGuard[StrOrBytesPath]:
-    """Checks if an object is a string, and that it points to a directory."""
-    return is_path(f) and os.path.isdir(f)
-
-
 class DeferredError:
     def __init__(self, ex: BaseException):
         self.ex = ex
